Documentation
¶
Index ¶
Constants ¶
This section is empty.
Variables ¶
This section is empty.
Functions ¶
func AddToolChainScriptConditions ¶
func AddToolChainScriptConditions(t *testing.T, conds map[string]script.Cond, goHostOS, goHostArch string)
AddToolChainScriptConditions accepts a script.Cond map and adds into it a set of commonly used conditions for doing toolchains testing, including whether the platform supports cgo, a buildmode condition, support for GOEXPERIMENT testing, etc. Callers must also pass in current GOHOSTOOS/GOHOSTARCH settings, since some of the conditions introduced can be influenced by them.
Types ¶
This section is empty.
Click to show internal directories.
Click to hide internal directories.